@@ -23,8 +23,9 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#include <config.h>
+#include "config.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,9 +39,9 @@
 #include <wayland-client.h>
 #include "shared/os-compatibility.h"
 #include "shared/zalloc.h"
-#include "xdg-shell-unstable-v5-client-protocol.h"
+#include "xdg-shell-client-protocol.h"
 #include "fullscreen-shell-unstable-v1-client-protocol.h"
-#include "scaler-client-protocol.h"
+#include "viewporter-client-protocol.h"
 
 int print_debug = 0;
 
@@ -49,8 +50,8 @@ struct display {
 	struct wl_registry *registry;
 	int compositor_version;
 	struct wl_compositor *compositor;
-	struct wl_scaler *scaler;
-	struct xdg_shell *shell;
+	struct wp_viewporter *viewporter;
+	struct xdg_wm_base *wm_base;
 	struct zwp_fullscreen_shell_v1 *fshell;
 	struct wl_shm *shm;
 	uint32_t formats;
@@ -72,11 +73,13 @@ struct window {
 	struct display *display;
 	int width, height, border;
 	struct wl_surface *surface;
-	struct wl_viewport *viewport;
+	struct wp_viewport *viewport;
 	struct xdg_surface *xdg_surface;
+	struct xdg_toplevel *xdg_toplevel;
 	struct wl_callback *callback;
 	struct buffer buffers[2];
 	struct buffer *prev_buffer;
+	bool wait_for_configure;
 
 	enum window_flags flags;
 	int scale;
@@ -92,6 +95,9 @@ struct window {
 
 static int running = 1;
 
+static void
+redraw(void *data, struct wl_callback *callback, uint32_t time);
+
 static void
 buffer_release(void *data, struct wl_buffer *buffer)
 {
@@ -143,21 +149,39 @@ create_shm_buffer(struct display *display, struct buffer *buffer,
 }
 
 static void
-handle_configure(void *data, struct xdg_surface *surface,
-		 int32_t width, int32_t height, struct wl_array *states,
-		 uint32_t serial)
+xdg_surface_handle_configure(void *data, struct xdg_surface *surface,
+			     uint32_t serial)
+{
+	struct window *window = data;
+
+	xdg_surface_ack_configure(surface, serial);
+
+	if (window->wait_for_configure) {
+		redraw(window, NULL, 0);
+		window->wait_for_configure = false;
+	}
+}
+
+static const struct xdg_surface_listener xdg_surface_listener = {
+	xdg_surface_handle_configure,
+};
+
+static void
+xdg_toplevel_handle_configure(void *data, struct xdg_toplevel *toplevel,
+			      int32_t width, int32_t height,
+			      struct wl_array *states)
 {
 }
 
 static void
-handle_close(void *data, struct xdg_surface *xdg_surface)
+xdg_toplevel_handle_close(void *data, struct xdg_toplevel *xdg_toplevel)
 {
 	running = 0;
 }
 
-static const struct xdg_surface_listener xdg_surface_listener = {
-	handle_configure,
-	handle_close,
+static const struct xdg_toplevel_listener xdg_toplevel_listener = {
+	xdg_toplevel_handle_configure,
+	xdg_toplevel_handle_close,
 };
 
 static float
@@ -258,8 +282,8 @@ create_window(struct display *display, int width, int height,
 		exit(1);
 	}
 
-	if (display->scaler == NULL && (flags & WINDOW_FLAG_USE_VIEWPORT)) {
-		fprintf(stderr, "Compositor does not support wl_viewport");
+	if (display->viewporter == NULL && (flags & WINDOW_FLAG_USE_VIEWPORT)) {
+		fprintf(stderr, "Compositor does not support wp_viewport");
 		exit(1);
 	}
 
@@ -290,20 +314,31 @@ create_window(struct display *display, int width, int height,
 	window->surface = wl_compositor_create_surface(display->compositor);
 
 	if (window->flags & WINDOW_FLAG_USE_VIEWPORT)
-		window->viewport = wl_scaler_get_viewport(display->scaler,
-							  window->surface);
+		window->viewport = wp_viewporter_get_viewport(display->viewporter,
+							      window->surface);
 
-	if (display->shell) {
+	if (display->wm_base) {
 		window->xdg_surface =
-			xdg_shell_get_xdg_surface(display->shell,
-						  window->surface);
+			xdg_wm_base_get_xdg_surface(display->wm_base,
+						    window->surface);
 
 		assert(window->xdg_surface);
 
 		xdg_surface_add_listener(window->xdg_surface,
 					 &xdg_surface_listener, window);
 
-		xdg_surface_set_title(window->xdg_surface, "simple-damage");
+		window->xdg_toplevel =
+			xdg_surface_get_toplevel(window->xdg_surface);
+
+		assert(window->xdg_toplevel);
+
+		xdg_toplevel_add_listener(window->xdg_toplevel,
+					  &xdg_toplevel_listener, window);
+
+		xdg_toplevel_set_title(window->xdg_toplevel, "simple-damage");
+
+		window->wait_for_configure = true;
+		wl_surface_commit(window->surface);
 	} else if (display->fshell) {
 		zwp_fullscreen_shell_v1_present_surface(display->fshell,
 							window->surface,
@@ -334,10 +369,12 @@ destroy_window(struct window *window)
 	if (window->buffers[1].buffer)
 		wl_buffer_destroy(window->buffers[1].buffer);
 
+	if (window->xdg_toplevel)
+		xdg_toplevel_destroy(window->xdg_toplevel);
 	if (window->xdg_surface)
 		xdg_surface_destroy(window->xdg_surface);
 	if (window->viewport)
-		wl_viewport_destroy(window->viewport);
+		wp_viewport_destroy(window->viewport);
 	wl_surface_destroy(window->surface);
 	free(window);
 }
@@ -560,7 +597,7 @@ redraw(void *data, struct wl_callback *callback, uint32_t time)
 			off_x = ty;
 			break;
 		}
-		wl_viewport_set_source(window->viewport,
+		wp_viewport_set_source(window->viewport,
 				       wl_fixed_from_int(window->width / 3),
 				       wl_fixed_from_int(window->height / 5),
 				       wl_fixed_from_int(window->width / 2),
@@ -640,7 +677,7 @@ redraw(void *data, struct wl_callback *callback, uint32_t time)
 						window->transform);
 
 	if (window->viewport)
-		wl_viewport_set_destination(window->viewport,
+		wp_viewport_set_destination(window->viewport,
 					    window->width,
 					    window->height);
 
@@ -674,21 +711,15 @@ struct wl_shm_listener shm_listener = {
 };
 
 static void
-xdg_shell_ping(void *data, struct xdg_shell *shell, uint32_t serial)
+xdg_wm_base_ping(void *data, struct xdg_wm_base *shell, uint32_t serial)
 {
-	xdg_shell_pong(shell, serial);
+	xdg_wm_base_pong(shell, serial);
 }
 
-static const struct xdg_shell_listener xdg_shell_listener = {
-	xdg_shell_ping,
+static const struct xdg_wm_base_listener wm_base_listener = {
+	xdg_wm_base_ping,
 };
 
-#define XDG_VERSION 5 /* The version of xdg-shell that we implement */
-#ifdef static_assert
-static_assert(XDG_VERSION == XDG_SHELL_VERSION_CURRENT,
-	      "Interface version doesn't match implementation version");
-#endif
-
 static void
 registry_handle_global(void *data, struct wl_registry *registry,
 		       uint32_t id, const char *interface, uint32_t version)
@@ -709,14 +740,13 @@ registry_handle_global(void *data, struct wl_registry *registry,
 			wl_registry_bind(registry,
 					 id, &wl_compositor_interface,
 					 d->compositor_version);
-	} else if (strcmp(interface, "wl_scaler") == 0 && version >= 2) {
-		d->scaler = wl_registry_bind(registry,
-					     id, &wl_scaler_interface, 2);
-	} else if (strcmp(interface, "xdg_shell") == 0) {
-		d->shell = wl_registry_bind(registry,
-					    id, &xdg_shell_interface, 1);
-		xdg_shell_use_unstable_version(d->shell, XDG_VERSION);
-		xdg_shell_add_listener(d->shell, &xdg_shell_listener, d);
+	} else if (strcmp(interface, "wp_viewporter") == 0) {
+		d->viewporter = wl_registry_bind(registry, id,
+						 &wp_viewporter_interface, 1);
+	} else if (strcmp(interface, "xdg_wm_base") == 0) {
+		d->wm_base = wl_registry_bind(registry,
+					      id, &xdg_wm_base_interface, 1);
+		xdg_wm_base_add_listener(d->wm_base, &wm_base_listener, d);
 	} else if (strcmp(interface, "zwp_fullscreen_shell_v1") == 0) {
 		d->fshell = wl_registry_bind(registry,
 					     id, &zwp_fullscreen_shell_v1_interface, 1);
@@ -778,14 +808,14 @@ destroy_display(struct display *display)
 	if (display->shm)
 		wl_shm_destroy(display->shm);
 
-	if (display->shell)
-		xdg_shell_destroy(display->shell);
+	if (display->wm_base)
+		xdg_wm_base_destroy(display->wm_base);
 
 	if (display->fshell)
 		zwp_fullscreen_shell_v1_release(display->fshell);
 
-	if (display->scaler)
-		wl_scaler_destroy(display->scaler);
+	if (display->viewporter)
+		wp_viewporter_destroy(display->viewporter);
 
 	if (display->compositor)
 		wl_compositor_destroy(display->compositor);
@@ -816,7 +846,7 @@ print_usage(int retval)
 		"  --scale=SCALE\t\tScale factor for the surface\n"
 		"  --transform=TRANSFORM\tTransform for the surface\n"
 		"  --rotating-transform\tUse a different buffer_transform for each frame\n"
-		"  --use-viewport\tUse wl_viewport\n"
+		"  --use-viewport\tUse wp_viewport\n"
 		"  --use-damage-buffer\tUse damage_buffer to post damage\n"
 	);
 
@@ -912,7 +942,8 @@ main(int argc, char **argv)
 	sigint.sa_flags = SA_RESETHAND;
 	sigaction(SIGINT, &sigint, NULL);
 
-	redraw(window, NULL, 0);
+	if (!window->wait_for_configure)
+		redraw(window, NULL, 0);
 
 	while (running && ret != -1)
 		ret = wl_display_dispatch(display->display);
