@@ -27,7 +27,7 @@
  * Implementation of ivi-layout library. The actual view on ivi_screen is
  * not updated until ivi_layout_commit_changes is called. An overview from
  * calling API for updating properties of ivi_surface/ivi_layer to asking
- * compositor to compose them by using weston_compositor_schedule_repaint,
+ * compositor to compose them by using weston_view_schedule_repaint,
  * 0/ initialize this library by ivi_layout_init_with_compositor
  *    with (struct weston_compositor *ec) from ivi-shell.
  * 1/ When an API for updating properties of ivi_surface/ivi_layer, it updates
@@ -42,8 +42,8 @@
  *    per each frame. See ivi-layout-transition.c in details. Transition
  *    animation interpolates frames between previous properties of ivi_surface
  *    and new ones.
- *    For example, when a property of ivi_surface is changed from invisibile
- *    to visibile, it behaves like fade-in. When ivi_layout_commitChange is
+ *    For example, when a property of ivi_surface is changed from invisible
+ *    to visible, it behaves like fade-in. When ivi_layout_commitChange is
  *    called during transition animation, it cancels the transition and
  *    re-start transition to new properties from current properties of final
  *    frame just before the cancellation.
@@ -51,8 +51,8 @@
  * 4/ According properties, set transformation by using weston_matrix and
  *    weston_view per ivi_surfaces and ivi_layers in while loop.
  * 5/ Set damage and trigger transform by using weston_view_geometry_dirty.
- * 6/ Notify update of properties.
- * 7/ Trigger composition by weston_compositor_schedule_repaint.
+ * 6/ Schedule repaint for each view by using weston_view_schedule_repaint.
+ * 7/ Notify update of properties.
  *
  */
 #include "config.h"
@@ -153,7 +153,10 @@ ivi_view_destroy(struct ivi_layout_view *ivi_view)
 	wl_list_remove(&ivi_view->pending_link);
 	wl_list_remove(&ivi_view->order_link);
 
-	weston_view_destroy(ivi_view->view);
+	if (weston_surface_is_desktop_surface(ivi_view->ivisurf->surface))
+		weston_desktop_surface_unlink_view(ivi_view->view);
+	else
+		weston_view_destroy(ivi_view->view);
 
 	free(ivi_view);
 }
@@ -170,9 +173,16 @@ ivi_view_create(struct ivi_layout_layer *ivilayer,
 		return NULL;
 	}
 
-	ivi_view->view = weston_view_create(ivisurf->surface);
+	if (weston_surface_is_desktop_surface(ivisurf->surface)) {
+		ivi_view->view = weston_desktop_surface_create_view(
+				ivisurf->weston_desktop_surface);
+	} else {
+		ivi_view->view = weston_view_create(ivisurf->surface);
+	}
+
 	if (ivi_view->view == NULL) {
 		weston_log("fails to allocate memory\n");
+		free(ivi_view);
 		return NULL;
 	}
 
@@ -320,72 +330,13 @@ update_opacity(struct ivi_layout_layer *ivilayer,
 	view->alpha = layer_alpha * surf_alpha;
 }
 
-static void
-get_rotate_values(enum wl_output_transform orientation,
-		  float *v_sin,
-		  float *v_cos)
-{
-	switch (orientation) {
-	case WL_OUTPUT_TRANSFORM_90:
-		*v_sin = 1.0f;
-		*v_cos = 0.0f;
-		break;
-	case WL_OUTPUT_TRANSFORM_180:
-		*v_sin = 0.0f;
-		*v_cos = -1.0f;
-		break;
-	case WL_OUTPUT_TRANSFORM_270:
-		*v_sin = -1.0f;
-		*v_cos = 0.0f;
-		break;
-	case WL_OUTPUT_TRANSFORM_NORMAL:
-	default:
-		*v_sin = 0.0f;
-		*v_cos = 1.0f;
-		break;
-	}
-}
-
-static void
-get_scale(enum wl_output_transform orientation,
-	  float dest_width,
-	  float dest_height,
-	  float source_width,
-	  float source_height,
-	  float *scale_x,
-	  float *scale_y)
-{
-	switch (orientation) {
-	case WL_OUTPUT_TRANSFORM_90:
-		*scale_x = dest_width / source_height;
-		*scale_y = dest_height / source_width;
-		break;
-	case WL_OUTPUT_TRANSFORM_180:
-		*scale_x = dest_width / source_width;
-		*scale_y = dest_height / source_height;
-		break;
-	case WL_OUTPUT_TRANSFORM_270:
-		*scale_x = dest_width / source_height;
-		*scale_y = dest_height / source_width;
-		break;
-	case WL_OUTPUT_TRANSFORM_NORMAL:
-	default:
-		*scale_x = dest_width / source_width;
-		*scale_y = dest_height / source_height;
-		break;
-	}
-}
-
 static void
 calc_transformation_matrix(struct ivi_rectangle *source_rect,
 			   struct ivi_rectangle *dest_rect,
-			   enum wl_output_transform orientation,
 			   struct weston_matrix *m)
 {
 	float source_center_x;
 	float source_center_y;
-	float vsin;
-	float vcos;
 	float scale_x;
 	float scale_y;
 	float translate_x;
@@ -395,16 +346,8 @@ calc_transformation_matrix(struct ivi_rectangle *source_rect,
 	source_center_y = source_rect->y + source_rect->height * 0.5f;
 	weston_matrix_translate(m, -source_center_x, -source_center_y, 0.0f);
 
-	get_rotate_values(orientation, &vsin, &vcos);
-	weston_matrix_rotate_xy(m, vcos, vsin);
-
-	get_scale(orientation,
-		  dest_rect->width,
-		  dest_rect->height,
-		  source_rect->width,
-		  source_rect->height,
-		  &scale_x,
-		  &scale_y);
+	scale_x = (float) dest_rect->width / (float) source_rect->width;
+	scale_y = (float) dest_rect->height / (float) source_rect->height;
 	weston_matrix_scale(m, scale_x, scale_y, 1.0f);
 
 	translate_x = dest_rect->width * 0.5f + dest_rect->x;
@@ -581,13 +524,8 @@ calc_surface_to_global_matrix_and_mask_to_weston_surface(
 	 * - single screen-local coordinates to multi-screen coordinates,
 	 *   which are global coordinates.
 	 */
-	calc_transformation_matrix(&surface_source_rect,
-				   &surface_dest_rect,
-				   sp->orientation, m);
-
-	calc_transformation_matrix(&layer_source_rect,
-				   &layer_dest_rect,
-				   lp->orientation, m);
+	calc_transformation_matrix(&surface_source_rect, &surface_dest_rect, m);
+	calc_transformation_matrix(&layer_source_rect, &layer_dest_rect, m);
 
 	weston_matrix_translate(m, output->x, output->y, 0.0f);
 
@@ -648,53 +586,37 @@ update_prop(struct ivi_layout_view *ivi_view)
 			       &ivi_view->transform.link);
 
 		weston_view_set_transform_parent(ivi_view->view, NULL);
+		weston_view_geometry_dirty(ivi_view->view);
+		weston_view_update_transform(ivi_view->view);
 	}
 
 	ivisurf->update_count++;
 
-	weston_view_geometry_dirty(ivi_view->view);
+	weston_view_schedule_repaint(ivi_view->view);
+}
 
-	weston_surface_damage(ivisurf->surface);
+static bool
+ivi_view_is_mapped(struct ivi_layout_view *ivi_view)
+{
+	return (!wl_list_empty(&ivi_view->order_link) &&
+		ivi_view->on_layer->on_screen &&
+		ivi_view->on_layer->prop.visibility &&
+		ivi_view->ivisurf->prop.visibility);
 }
 
 static void
 commit_changes(struct ivi_layout *layout)
 {
-	struct ivi_layout_screen *iviscrn = NULL;
-	struct ivi_layout_layer *ivilayer = NULL;
-	struct ivi_layout_surface *ivisurf = NULL;
 	struct ivi_layout_view *ivi_view  = NULL;
 
 	wl_list_for_each(ivi_view, &layout->view_list, link) {
-		ivisurf = ivi_view->ivisurf;
-		ivilayer = ivi_view->on_layer;
-		iviscrn = ivilayer->on_screen;
-
 		/*
 		 * If the view is not on the currently rendered scenegraph,
 		 * we do not need to update its properties.
 		 */
-		if (wl_list_empty(&ivi_view->order_link) || !iviscrn)
+		if (!ivi_view_is_mapped(ivi_view))
 			continue;
 
-		/*
-		 * If the view's layer or surface is invisible, we do not need
-		 * to update its properties.
-		 */
-		if (!ivilayer->prop.visibility || !ivisurf->prop.visibility) {
-			/*
-			* If ivilayer or ivisurf of ivi_view is made invisible
-			* in this commit_changes call, we have to damage
-			* the weston_view below this ivi_view. Otherwise content
-			* of this ivi_view will stay visible.
-			*/
-			if ((ivilayer->prop.event_mask | ivisurf->prop.event_mask) &&
-			    IVI_NOTIFICATION_VISIBILITY)
-				weston_view_damage_below(ivi_view->view);
-
-			continue;
-		}
-
 		update_prop(ivi_view);
 	}
 }
@@ -777,9 +699,15 @@ commit_surface_list(struct ivi_layout *layout)
 			ivisurf->pending.prop.transition_type = IVI_LAYOUT_TRANSITION_NONE;
 
 			if (configured && !is_surface_transition(ivisurf)) {
-				shell_surface_send_configure(ivisurf->surface,
-							     ivisurf->prop.dest_width,
-							     ivisurf->prop.dest_height);
+				if (weston_surface_is_desktop_surface(ivisurf->surface)) {
+					weston_desktop_surface_set_size(ivisurf->weston_desktop_surface,
+									ivisurf->prop.dest_width,
+									ivisurf->prop.dest_height);
+				} else {
+					shell_surface_send_configure(ivisurf->surface,
+								     ivisurf->prop.dest_width,
+								     ivisurf->prop.dest_height);
+				}
 			}
 		} else {
 			configured = 0;
@@ -851,10 +779,6 @@ commit_screen_list(struct ivi_layout *layout)
 	struct ivi_layout_screen  *iviscrn  = NULL;
 	struct ivi_layout_layer   *ivilayer = NULL;
 	struct ivi_layout_layer   *next     = NULL;
-	struct ivi_layout_view *ivi_view = NULL;
-
-	/* Clear view list of layout ivi_layer */
-	wl_list_init(&layout->layout_layer.view_list.link);
 
 	wl_list_for_each(iviscrn, &layout->screen_list, link) {
 		if (iviscrn->order.dirty) {
@@ -881,7 +805,28 @@ commit_screen_list(struct ivi_layout *layout)
 
 			iviscrn->order.dirty = 0;
 		}
+	}
+}
 
+static void
+build_view_list(struct ivi_layout *layout)
+{
+	struct ivi_layout_screen  *iviscrn;
+	struct ivi_layout_layer   *ivilayer;
+	struct ivi_layout_view   *ivi_view;
+
+	/* If ivi_view is not part of the scenegrapgh, we have to unmap
+	 * weston_views
+	 */
+	wl_list_for_each(ivi_view, &layout->view_list, link) {
+		if (!ivi_view_is_mapped(ivi_view))
+			weston_view_unmap(ivi_view->view);
+	}
+
+	/* Clear view list of layout ivi_layer */
+	wl_list_init(&layout->layout_layer.view_list.link);
+
+	wl_list_for_each(iviscrn, &layout->screen_list, link) {
 		wl_list_for_each(ivilayer, &iviscrn->order.layer_list, order.link) {
 			if (ivilayer->prop.visibility == false)
 				continue;
@@ -893,7 +838,6 @@ commit_screen_list(struct ivi_layout *layout)
 				weston_layer_entry_insert(&layout->layout_layer.view_list,
 							  &ivi_view->view->layer_link);
 
-				ivi_view->view->output = iviscrn->output;
 				ivi_view->ivisurf->surface->is_mapped = true;
 				ivi_view->view->is_mapped = true;
 			}
@@ -1039,6 +983,21 @@ ivi_layout_add_listener_configure_surface(struct wl_listener *listener)
 	return IVI_SUCCEEDED;
 }
 
+static int32_t
+ivi_layout_add_listener_configure_desktop_surface(struct wl_listener *listener)
+{
+	struct ivi_layout *layout = get_instance();
+
+	if (!listener) {
+		weston_log("ivi_layout_add_listener_configure_desktop_surface: invalid argument\n");
+		return IVI_FAILED;
+	}
+
+	wl_signal_add(&layout->surface_notification.configure_desktop_changed, listener);
+
+	return IVI_SUCCEEDED;
+}
+
 uint32_t
 ivi_layout_get_id_of_surface(struct ivi_layout_surface *ivisurf)
 {
@@ -1488,28 +1447,6 @@ ivi_layout_layer_set_destination_rectangle(struct ivi_layout_layer *ivilayer,
 	return IVI_SUCCEEDED;
 }
 
-static int32_t
-ivi_layout_layer_set_orientation(struct ivi_layout_layer *ivilayer,
-				 enum wl_output_transform orientation)
-{
-	struct ivi_layout_layer_properties *prop = NULL;
-
-	if (ivilayer == NULL) {
-		weston_log("ivi_layout_layer_set_orientation: invalid argument\n");
-		return IVI_FAILED;
-	}
-
-	prop = &ivilayer->pending.prop;
-	prop->orientation = orientation;
-
-	if (ivilayer->prop.orientation != orientation)
-		prop->event_mask |= IVI_NOTIFICATION_ORIENTATION;
-	else
-		prop->event_mask &= ~IVI_NOTIFICATION_ORIENTATION;
-
-	return IVI_SUCCEEDED;
-}
-
 int32_t
 ivi_layout_layer_set_render_order(struct ivi_layout_layer *ivilayer,
 				  struct ivi_layout_surface **pSurface,
@@ -1619,28 +1556,6 @@ ivi_layout_surface_set_destination_rectangle(struct ivi_layout_surface *ivisurf,
 	return IVI_SUCCEEDED;
 }
 
-static int32_t
-ivi_layout_surface_set_orientation(struct ivi_layout_surface *ivisurf,
-				   enum wl_output_transform orientation)
-{
-	struct ivi_layout_surface_properties *prop = NULL;
-
-	if (ivisurf == NULL) {
-		weston_log("ivi_layout_surface_set_orientation: invalid argument\n");
-		return IVI_FAILED;
-	}
-
-	prop = &ivisurf->pending.prop;
-	prop->orientation = orientation;
-
-	if (ivisurf->prop.orientation != orientation)
-		prop->event_mask |= IVI_NOTIFICATION_ORIENTATION;
-	else
-		prop->event_mask &= ~IVI_NOTIFICATION_ORIENTATION;
-
-	return IVI_SUCCEEDED;
-}
-
 static int32_t
 ivi_layout_screen_add_layer(struct weston_output *output,
 			    struct ivi_layout_layer *addlayer)
@@ -1654,6 +1569,11 @@ ivi_layout_screen_add_layer(struct weston_output *output,
 
 	iviscrn = get_screen_from_output(output);
 
+	/*if layer is already assigned to screen make order of it dirty
+	 * we are going to remove it (in commit_screen_list)*/
+	if (addlayer->on_screen)
+		addlayer->on_screen->order.dirty = 1;
+
 	wl_list_remove(&addlayer->pending.link);
 	wl_list_insert(&iviscrn->pending.layer_list, &addlayer->pending.link);
 
@@ -1662,6 +1582,27 @@ ivi_layout_screen_add_layer(struct weston_output *output,
 	return IVI_SUCCEEDED;
 }
 
+static int32_t
+ivi_layout_screen_remove_layer(struct weston_output *output,
+			    struct ivi_layout_layer *removelayer)
+{
+	struct ivi_layout_screen *iviscrn;
+
+	if (output == NULL || removelayer == NULL) {
+		weston_log("ivi_layout_screen_remove_layer: invalid argument\n");
+		return IVI_FAILED;
+	}
+
+	iviscrn = get_screen_from_output(output);
+
+	wl_list_remove(&removelayer->pending.link);
+	wl_list_init(&removelayer->pending.link);
+
+	iviscrn->order.dirty = 1;
+
+	return IVI_SUCCEEDED;
+}
+
 static int32_t
 ivi_layout_screen_set_render_order(struct weston_output *output,
 				   struct ivi_layout_layer **pLayer,
@@ -1841,12 +1782,12 @@ ivi_layout_commit_changes(void)
 	commit_surface_list(layout);
 	commit_layer_list(layout);
 	commit_screen_list(layout);
+	build_view_list(layout);
 
 	commit_transition(layout);
 
 	commit_changes(layout);
 	send_prop(layout);
-	weston_compositor_schedule_repaint(layout->compositor);
 
 	return IVI_SUCCEEDED;
 }
@@ -1900,6 +1841,44 @@ ivi_layout_surface_set_transition_duration(struct ivi_layout_surface *ivisurf,
 	return 0;
 }
 
+/*
+ * This interface enables e.g. an id agent to set the id of an ivi-layout
+ * surface, that has been created by a desktop application. This can only be
+ * done once as long as the initial surface id equals IVI_INVALID_ID. Afterwards
+ * two events are emitted, namely surface_created and surface_configured.
+ */
+static int32_t
+ivi_layout_surface_set_id(struct ivi_layout_surface *ivisurf,
+			  uint32_t id_surface)
+{
+	struct ivi_layout *layout = get_instance();
+	struct ivi_layout_surface *search_ivisurf = NULL;
+
+	if (!ivisurf) {
+		weston_log("%s: invalid argument\n", __func__);
+		return IVI_FAILED;
+	}
+
+	if (ivisurf->id_surface != IVI_INVALID_ID) {
+		weston_log("surface id can only be set once\n");
+		return IVI_FAILED;
+	}
+
+	search_ivisurf = get_surface(&layout->surface_list, id_surface);
+	if (search_ivisurf) {
+		weston_log("id_surface(%d) is already created\n", id_surface);
+		return IVI_FAILED;
+	}
+
+	ivisurf->id_surface = id_surface;
+
+	wl_signal_emit(&layout->surface_notification.created, ivisurf);
+	wl_signal_emit(&layout->surface_notification.configure_changed,
+		       ivisurf);
+
+	return IVI_SUCCEEDED;
+}
+
 static int32_t
 ivi_layout_surface_set_transition(struct ivi_layout_surface *ivisurf,
 				  enum ivi_layout_transition_type type,
@@ -1941,20 +1920,8 @@ ivi_layout_surface_dump(struct weston_surface *surface,
  * methods of interaction between ivi-shell with ivi-layout
  */
 
-void
-ivi_layout_surface_configure(struct ivi_layout_surface *ivisurf,
-			     int32_t width, int32_t height)
-{
-	struct ivi_layout *layout = get_instance();
-
-	/* emit callback which is set by ivi-layout api user */
-	wl_signal_emit(&layout->surface_notification.configure_changed,
-		       ivisurf);
-}
-
-struct ivi_layout_surface*
-ivi_layout_surface_create(struct weston_surface *wl_surface,
-			  uint32_t id_surface)
+static struct ivi_layout_surface*
+surface_create(struct weston_surface *wl_surface, uint32_t id_surface)
 {
 	struct ivi_layout *layout = get_instance();
 	struct ivi_layout_surface *ivisurf = NULL;
@@ -1964,14 +1931,6 @@ ivi_layout_surface_create(struct weston_surface *wl_surface,
 		return NULL;
 	}
 
-	ivisurf = get_surface(&layout->surface_list, id_surface);
-	if (ivisurf != NULL) {
-		if (ivisurf->surface != NULL) {
-			weston_log("id_surface(%d) is already created\n", id_surface);
-			return NULL;
-		}
-	}
-
 	ivisurf = calloc(1, sizeof *ivisurf);
 	if (ivisurf == NULL) {
 		weston_log("fails to allocate memory\n");
@@ -1995,11 +1954,60 @@ ivi_layout_surface_create(struct weston_surface *wl_surface,
 
 	wl_list_insert(&layout->surface_list, &ivisurf->link);
 
-	wl_signal_emit(&layout->surface_notification.created, ivisurf);
+	return ivisurf;
+}
+
+void
+ivi_layout_desktop_surface_configure(struct ivi_layout_surface *ivisurf,
+				 int32_t width, int32_t height)
+{
+	struct ivi_layout *layout = get_instance();
+
+	/* emit callback which is set by ivi-layout api user */
+	wl_signal_emit(&layout->surface_notification.configure_desktop_changed,
+		       ivisurf);
+}
+
+struct ivi_layout_surface*
+ivi_layout_desktop_surface_create(struct weston_surface *wl_surface)
+{
+	return surface_create(wl_surface, IVI_INVALID_ID);
+}
+
+void
+ivi_layout_surface_configure(struct ivi_layout_surface *ivisurf,
+			     int32_t width, int32_t height)
+{
+	struct ivi_layout *layout = get_instance();
+
+	/* emit callback which is set by ivi-layout api user */
+	wl_signal_emit(&layout->surface_notification.configure_changed,
+		       ivisurf);
+}
+
+struct ivi_layout_surface*
+ivi_layout_surface_create(struct weston_surface *wl_surface,
+			  uint32_t id_surface)
+{
+	struct ivi_layout *layout = get_instance();
+	struct ivi_layout_surface *ivisurf = NULL;
+
+	ivisurf = get_surface(&layout->surface_list, id_surface);
+	if (ivisurf) {
+		weston_log("id_surface(%d) is already created\n", id_surface);
+		return NULL;
+	}
+
+	ivisurf = surface_create(wl_surface, id_surface);
+
+	if (ivisurf)
+		wl_signal_emit(&layout->surface_notification.created, ivisurf);
 
 	return ivisurf;
 }
 
+static struct ivi_layout_interface ivi_layout_interface;
+
 void
 ivi_layout_init_with_compositor(struct weston_compositor *ec)
 {
@@ -2018,6 +2026,7 @@ ivi_layout_init_with_compositor(struct weston_compositor *ec)
 	wl_signal_init(&layout->surface_notification.created);
 	wl_signal_init(&layout->surface_notification.removed);
 	wl_signal_init(&layout->surface_notification.configure_changed);
+	wl_signal_init(&layout->surface_notification.configure_desktop_changed);
 
 	/* Add layout_layer at the last of weston_compositor.layer_list */
 	weston_layer_init(&layout->layout_layer, ec);
@@ -2028,6 +2037,10 @@ ivi_layout_init_with_compositor(struct weston_compositor *ec)
 
 	layout->transitions = ivi_layout_transition_set_create(ec);
 	wl_list_init(&layout->pending_transition_list);
+
+	weston_plugin_api_register(ec, IVI_LAYOUT_API_NAME,
+				   &ivi_layout_interface,
+				   sizeof(struct ivi_layout_interface));
 }
 
 static struct ivi_layout_interface ivi_layout_interface = {
@@ -2042,6 +2055,7 @@ static struct ivi_layout_interface ivi_layout_interface = {
 	.add_listener_create_surface	= ivi_layout_add_listener_create_surface,
 	.add_listener_remove_surface	= ivi_layout_add_listener_remove_surface,
 	.add_listener_configure_surface	= ivi_layout_add_listener_configure_surface,
+	.add_listener_configure_desktop_surface	= ivi_layout_add_listener_configure_desktop_surface,
 	.get_surface				= shell_get_ivi_layout_surface,
 	.get_surfaces				= ivi_layout_get_surfaces,
 	.get_id_of_surface			= ivi_layout_get_id_of_surface,
@@ -2052,11 +2066,11 @@ static struct ivi_layout_interface ivi_layout_interface = {
 	.surface_set_opacity			= ivi_layout_surface_set_opacity,
 	.surface_set_source_rectangle		= ivi_layout_surface_set_source_rectangle,
 	.surface_set_destination_rectangle	= ivi_layout_surface_set_destination_rectangle,
-	.surface_set_orientation		= ivi_layout_surface_set_orientation,
 	.surface_add_listener			= ivi_layout_surface_add_listener,
 	.surface_get_weston_surface		= ivi_layout_surface_get_weston_surface,
 	.surface_set_transition			= ivi_layout_surface_set_transition,
 	.surface_set_transition_duration	= ivi_layout_surface_set_transition_duration,
+	.surface_set_id				= ivi_layout_surface_set_id,
 
 	/**
 	 * layer controller interfaces
@@ -2075,7 +2089,6 @@ static struct ivi_layout_interface ivi_layout_interface = {
 	.layer_set_opacity			= ivi_layout_layer_set_opacity,
 	.layer_set_source_rectangle		= ivi_layout_layer_set_source_rectangle,
 	.layer_set_destination_rectangle	= ivi_layout_layer_set_destination_rectangle,
-	.layer_set_orientation			= ivi_layout_layer_set_orientation,
 	.layer_add_surface			= ivi_layout_layer_add_surface,
 	.layer_remove_surface			= ivi_layout_layer_remove_surface,
 	.layer_set_render_order			= ivi_layout_layer_set_render_order,
@@ -2087,6 +2100,7 @@ static struct ivi_layout_interface ivi_layout_interface = {
 	 */
 	.get_screens_under_layer	= ivi_layout_get_screens_under_layer,
 	.screen_add_layer		= ivi_layout_screen_add_layer,
+	.screen_remove_layer		= ivi_layout_screen_remove_layer,
 	.screen_set_render_order	= ivi_layout_screen_set_render_order,
 
 	/**
@@ -2101,43 +2115,3 @@ static struct ivi_layout_interface ivi_layout_interface = {
 	.surface_get_size		= ivi_layout_surface_get_size,
 	.surface_dump			= ivi_layout_surface_dump,
 };
-
-int
-load_controller_modules(struct weston_compositor *compositor, const char *modules,
-			int *argc, char *argv[])
-{
-	const char *p, *end;
-	char buffer[256];
-	int (*controller_module_init)(struct weston_compositor *compositor,
-				      int *argc, char *argv[],
-				      const struct ivi_layout_interface *interface,
-				      size_t interface_version);
-
-	if (modules == NULL)
-		return 0;
-
-	p = modules;
-	while (*p) {
-		end = strchrnul(p, ',');
-		snprintf(buffer, sizeof buffer, "%.*s", (int)(end - p), p);
-
-		controller_module_init =
-			wet_load_module_entrypoint(buffer,
-						   "controller_module_init");
-		if (!controller_module_init)
-			return -1;
-
-		if (controller_module_init(compositor, argc, argv,
-					   &ivi_layout_interface,
-				sizeof(struct ivi_layout_interface)) != 0) {
-			weston_log("ivi-shell: Initialization of controller module fails");
-			return -1;
-		}
-
-		p = end;
-		while (*p == ',')
-			p++;
-	}
-
-	return 0;
-}
