@@ -28,7 +28,8 @@
 #include <stdio.h>
 #include <assert.h>
 
-#include "src/compositor.h"
+#include "compositor.h"
+#include "compositor/weston.h"
 
 static void
 surface_transform(void *data)
@@ -56,11 +57,12 @@ surface_transform(void *data)
 	weston_view_to_global_float(view, 50, 40, &x, &y);
 	assert(x == 200 && y == 340);
 
-	wl_display_terminate(compositor->wl_display);
+	weston_compositor_exit(compositor);
 }
 
 WL_EXPORT int
-module_init(struct weston_compositor *compositor, int *argc, char *argv[])
+wet_module_init(struct weston_compositor *compositor,
+		int *argc, char *argv[])
 {
 	struct wl_event_loop *loop;
 
